package btree.visitor;
import btree.*;
import java.util.*;

/**
 * Algorithm to insert a given new data element into a 
 * search tree as per the search criteria.
 * The ordering is determined by a Comparator object
 */
public class InsertSearchTree implements IBTreeAlgo {
  /**
   * The order (max # of data elements) of the tree
   */
  private int _order;
  
  /**
   * The factory used to instantiate new B-Trees when necessary
   */
  private IBTreeFac _fac;
  
  /**
   * The Comparator object used to determine the ordering of data 
   * elements.  By default is compares two Comparable objects.
   */
  private Comparator _comp = new Comparator() {
    public int compare(Object x, Object y) {
      return ((Comparable)x).compareTo(y);
    }
  };
  
  /**
   * Convenience value that references an empty tree
   * This value is initialize in the constructor using the 
   * IBTreeFac instance.
   */
  private IMTBTree _mt;
  
  /**
   * Constructor for the class that uses the default Comparator instance.
   * @param order The order of the B-Tree that is created.
   * @param fac The factory used to create the new trees.
   */
  public InsertSearchTree(int order, IBTreeFac fac){
    _order = order;
    _fac = fac;
    _mt = fac.makeMTBTree();
  }
  
  /**
   * Constructor for the class that uses the given Comparator instance.
   * @param order The order of the B-Tree that is created.
   * @param fac The factory used to create the new trees.
   * @param comp The Comparator object to use to determine ordering.
   */
  public InsertSearchTree(int order, IBTreeFac fac, Comparator comp){
    this(order, fac);
    _comp = comp;
  }
  
  /**
   * Tree is empty.  Return a new INEBTree that has exactly one data 
   * element, params[0], and whose children are all empty.
   * @param params params[0] is the new data element to store.
   * @return A new INEBTree instance.
   */
  public Object mtCase(IMTBTree host, Object...params) {
    // STUDENT TO COMPLETE

    return null;  // STUB CODE
  }
  
  /**
   * Tree is non-empty. 
   * If the number of root data elements is less than the order 
   * and the child tree at the insertion point is empty, then insert
   * the new data element into this node's data and add an empty 
   * child tree.  
   * If the number of root data elements is less than the order 
   * and the child tree at the insertion point is NOT empty,
   * insert the data element into the child tree.
   * If the number of root data elements is equal to 
   * the order, then always insert into the child tree at 
   * the insertion point.
   * A NEW INEBTree is ALWAYS returned because the tree is
   * immutable.
   * @param params params[0] is the new data element to store.
   * @return A new INEBTree instance.
   */
  public Object neCase(INEBTree host, final Object...params){
    // STUDENT TO COMPLETE

    return null;  // STUB CODE
  }
  
  /**
   * Finds the index of the child tree where a new data element 
   * would be inserted with respect to a given data array, which 
   * is assumed to be sorted in ascending order as per the _comp object.
   * 0 returned means that x is smaller than the first element in array.
   * array.length returned means that x is larger than the last element in array.
   */
  public int getInsertIndex(Object[] array, Object x) {
    // STUDENT TO COMPLETE
    return -1;  // STUB CODE
  }
}